"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var React = require("react");
var utilities_1 = require("./utilities");
/**
 * Assembles a higher order component based on the following: styles, theme, view, and state.
 * Imposes a separation of concern and centralizes styling processing to increase ease of use and robustness
 * in how components use and apply styling and theming.
 *
 * Automatically merges and applies themes and styles with theme / styleprops having the highest priority.
 * State component, if provided, is passed in props for processing. Props from state / user are automatically processed
 * and styled before finally being passed to view.
 *
 * State components should contain all stateful behavior and should not generate any JSX, but rather simply call the view prop.
 * Views should simply be stateless pure functions that receive all props needed for rendering their output.
 * State component is optional. If state not provided, created component is essentially a functional stateless component.
 *
 * TComponentProps: A styleable props interface for the created component.
 * TViewProps: The props specific to the view, including processed properties outputted by optional state component. If state
 * component is not provided, TComponentProps is the same as TViewProps.
 * TStyleSet: The type for styles properties.
 * TProcessedStyleSet: The type provided by mergeStyleSets provider after processing TStyleSet and provided to views.
 * TTheme: The type for theme properties as well as the getTheme provider.
 *
 * @param {IComponentOptions} options
 * @param {IStylingProviders} providers
 * @param {IStateComponent} StateComponent
 *
 * If your package has common types for any of the type arguments, such as TTheme and TProcessedStyleSet, it is strongly
 * recommended to make an interface file for your package that reduces the number of types individual components need
 * to provide. For example:
 * @example
 * export type IViewProps<TProps, TStyleSet extends IStyleSet<TStyleSet>> = IViewProps<TProps, IProcessedStyleSet<TStyleSet>>;
 * export type IStyleableComponent<TProps, TStyleSet> = IStyleableComponent<TProps, TStyleSet, ITheme>;
 *
 */
function createComponent(options, providers) {
    var CustomizerContext = providers.CustomizerContext;
    var result = function (userProps) {
        return (React.createElement(CustomizerContext.Consumer, null, function (context) {
            var settings = providers.getCustomizations(options.displayName, context);
            var _a = settings, contextStyles = _a.styles, rest = tslib_1.__rest(_a, ["styles"]);
            var renderView = function (processedProps) {
                // The approach here is to allow state components to provide only the props they care about, automatically
                //    merging user props and processed props together. This ensures all props are passed properly to view,
                //    including children and styles.
                // TODO: userProps should only override processedProps for controlled props. for all other props, such as callbacks,
                //    processedProps should always have priority (this is not the case as written now.)
                //    introduce controlled prop marking mechanism so that only controlled userProps override processedProps.
                // TODO: Should 'rest' props from customizations pass onto view? They're not currently. (props like theme will break snapshots)
                var propStyles = processedProps.styles || userProps.styles;
                var styleProps = tslib_1.__assign({}, rest, processedProps, userProps);
                var viewProps = tslib_1.__assign({}, processedProps, userProps, {
                    classNames: providers.mergeStyleSets(_evaluateStyle(styleProps, options.styles), _evaluateStyle(styleProps, contextStyles), _evaluateStyle(styleProps, propStyles))
                });
                // TODO: consider rendering view as JSX component with display name in debug mode to aid in debugging
                return options.view(viewProps);
            };
            // TODO: What we really need to be able to do here either type force TViewProps to be TComponentProps when StateComponent
            //        is undefined OR logically something like code below. Until we figure out how to do this, cast userProps as any
            //        since userProps does not necessarily extend TViewProps.
            // if (StateComponent) {
            //   type TProcessedProps = TViewProps & IStyleableComponentProps<TViewProps, TStyleSet, TTheme>;
            // } else {
            //   type TProcessedProps = TComponentProps & IStyleableComponentProps<TComponentProps, TStyleSet, TTheme>;
            // }
            return options.state ? React.createElement(options.state, tslib_1.__assign({}, userProps, { renderView: renderView })) : renderView(userProps);
        }));
    };
    result.displayName = options.displayName;
    utilities_1.assign(result, options.statics);
    // Later versions of TypeSript should allow us to merge objects in a type safe way and avoid this cast.
    return result;
}
exports.createComponent = createComponent;
/**
 * Evaluate styles based on type to return consistent TStyleSet.
 */
function _evaluateStyle(props, styles) {
    if (typeof styles === 'function') {
        return styles(props);
    }
    return styles;
}
//# sourceMappingURL=createComponent.js.map